#!/usr/bin/perl 

#overlap_stats.cgi -Calculate the probability of the overlap of x genes
#		    from two groups D and n drawn from N genes.
#
#Copyright (C) 2005  Jim Lund
#
#---------------------------
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#---------------------------
#Version: 0.011
#
#Last modified:  4/05
#Written by Jim Lund, Department of Biology, University of Kentucky.
#jiml at uky dot edu
#Web site: http://elegans.uky.edu
#---------------------------
#
#HISTORY 
#v.011, 4-5-05
#-'0' overlap treated as bad input, now OK.
#
#v.010, 4-5-05
#-Made hypergeometric probability calcs more accurate by switching to Gsl 
# library and Math::Gsl::Sf module.
#


use CGI qw(:standard);
use Math::Gsl::Sf qw(:Gamma);


#
#Subroutines
#
sub PARSE_FORM;
sub SELECT_GENES;
sub ACE_INFO;
sub PRINT_OUTPUT;
sub HTML_OUTPUT;
sub COMBINE_LISTS;
sub LOAD_LISTS3;
sub GET_DIR2;
sub SORT_LISTS2;
sub SORT_LISTS3;
sub GET_PERMS;
sub FORM_ERROR;
sub GET_LIST_FILES;
sub ET_INPUT_LIST;
sub GET_FORM_FILE2;
sub GET_SET;




#
#Global variables.
#
my $form_file = 'overlap_stats_tmpl.html';
my $form_page = "http://elegans.uky.edu/MA/progs/overlap_stats.html";



#
#Send out output without delay.
#
$|=1;



#
#Read in form values.
#
$page_result= new CGI;

$set1= $page_result->param('set1');
$set2= $page_result->param('set2');
$overlap= $page_result->param('overlap');
$genome= $page_result->param('genome');

#
#Sanity check on the input.
#
if ($set1 <= 0 || $set2 <= 0 || $overlap < 0 || $genome <= 0) {
    $query = new CGI;
    print $query->redirect($form_page);
    exit;
}


$rep_factor=sprintf("%.1f",($overlap/($set1*$set2/$genome)));


#
#Calculate probability.
#
$prob=normal_cum_prob($genome,$set1,$set2,$overlap);
if ($prob < .001) { $prob=sprintf("%.3e",$prob); }
else { $prob=sprintf("%.3f",$prob); }



#
#Read in template file.
#
if ($form_file) {
    (open(FORM,'<'.$form_file)) || die "Can't open file $form_file: $!\n";
    @page=<FORM>;
    close FORM || warn "Warning, error closing $form_file: $!\n";
    }
else {die "Can't find the form file!\n";}




#
#Now iterate through the form, replacing the active sections with the imported/
#calculated content.
#
for ($i=0;$i<=$#page;$i++) {
#
#Add in left side menu.
#
    if ($page[$i]=~/#set1#/i) {$page[$i]=~s/#set1#/$set1/i;}
    if ($page[$i]=~/#set2#/i) {$page[$i]=~s/#set2#/$set2/i;}
    if ($page[$i]=~/#overlap#/i) {$page[$i]=~s/#overlap#/$overlap/i;}
    if ($page[$i]=~/#genome#/i) {$page[$i]=~s/#genome#/$genome/i;}
    if ($page[$i]=~/#rep_factor#/i) {$page[$i]=~s/#rep_factor#/$rep_factor/i;}
    if ($page[$i]=~/#prob#/i) {$page[$i]=~s/#prob#/$prob/i;}
    }


#
#Print out page.
#
print header();
print @page;
exit;



#
#End of main section.
#


#
#Subroutines
#



############################################################################
#Called by: main
#Calls: lookup_hyper_prob, significance_P, erff
#
#Calculate the probability for the intersection of two
#groups of genes.
#
#               programs to compute exact values for the C. elegans paper
#               See Statistics In Research, by Bernard Ostel, Iowa State, 1963#               pages 73 and following.
#
#;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#       If there are N $total $genes
#       and          n $genes in a particular mountain
#       and          D $genes of a known kind in the $total of all $genes
#       and if       x is the number of those D particular $genes in this
#                       mountain,
#       then what is the probability of observing exactly x of them?
#
#       let C(a,b) be notation for a things taken b at a time}
#           C(a,b) = a! /(b! * (a-b)!)
#
#
#       Pr(x) = C(D, x) * C(N-D, n-x) / C(N,n)
#
#
#       The $expected number of marked $genes, x, found in the mountain is
#       E[X] = nD/N for a random drawing of size n form this
#       population of N $total $genes with D marked ones
#
#       the variance, E[(x-mu)^2] = nD(N-D)(N-n)/N^2(N-1), see page 74
#
#       When n is much bigger than D and when n<<N (say by a factor of 8
#       to 20)
#       the exact probability can be approximated using a normal distribution
#
#       Using the Normal distribution to compute a p (or alpha) value for
#       observing b or fewer marked $genes in a mountain.
#
#               (approx.)
#       P{X<=b}    =      P{ Z < ((b-0.5) - np ) / sqrt(np(1-p))  }
#       where n is number of $genes in the mountain,
#             p = D/N
#       and where Z is a standard normal variate from N(0,1)
#
#
#       For example, suppose there were N=17,661 $total $genes
#       and in a mountain with n=810 $genes, we find x=8 of D=64 marked $genes#               with p=64/17661 ~ 0.00362
#                    q=(1-p)    ~ 0.99638
#       the probability of observing fewer than the 8 marked $genes is
#
#       P(x<8) = P{Z < ( 7.5 -  2.94)/sqrt (810 * 0.0363 * 0.99638) }
#              = p{Z <  4.56 / 1.71 =  2.666} or about p=0.996174
#
#
#
#Find the probability for this overlap x of subsets n and D of the set N.
#
#Look it up in a table if this a pre-computed value.  Else compute the the 
#normal approx. to the hypergeometric prob., or the exact hypergeometric 
#prob. if the approx. won't be good enough.
#
sub normal_cum_prob {
my ($N,$n,$D,$x)=@_;

my ($p,$cp);
my $min=1e-20;

#
#Check for error.
#
if ($D == 0 || $n == 0) { return 1; }

my $rf=($x * $N) / ($D * $n);

#
#Calculate the cumulative hypergeometric probability.
#
my $Nn=Math::Gsl::Sf::lnchoose($N,$n);

if ($rf < 1) {
   for (;$x >= 0;$x--) {
        $p = exp(Math::Gsl::Sf::lnchoose($D,$x) + Math::Gsl::Sf::lnchoose($N-$D,$n-$x) - $Nn);
        $cp += $p;
        if ($p < $min) { last; }
        }
    }else{
   my $top = $D > $n ? $n : $D;
   for (;$x <= $top;$x++) {
        $p = exp(Math::Gsl::Sf::lnchoose($D,$x) + Math::Gsl::Sf::lnchoose($N-$D,$n-$x) - $Nn);
        $cp += $p;
        if ($p < $min) { last; }
        }
    }


#
#Make the p value always be < 0.5.
#
if ($cp > .5) { $cp=1-$cp; }

return $cp;
}


#
#End of program.




